/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : lpc2103.h																					
| Author			    : Pride Embedded, LLC																						
| Creation Date		: 6/22/06																							
| Version			    : 1.1																																												
| Spaces per tab	: 2																									
| Description		  : Header file for the Philips LPC2101/02/02 ARM microcontrollers 																																									 : 
| Revision			  : 1.0: Initial
|                 : 1.1: Added SCB_APBDIV define 																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

#ifndef lpc2103_h
#define lpc2103_h

/*
|----------------------------------------------------
| Terms of use
|----------------------------------------------------
*/

/*******************************************************************************
THE SOFTWARE IS DELIVERED "AS IS" WITHOUT WARRANTY OR CONDITION OF ANY KIND, 
EITHER EXPRESS, IMPLIED OR STATUTORY. THIS INCLUDES WITHOUT LIMITATION ANY 
WARRANTY OR CONDITION WITH RESPECT TO MERCHANTABILITY OR FITNESS FOR ANY 
PARTICULAR PURPOSE, OR AGAINST THE INFRINGEMENTS OF INTELLECTUAL PROPERTY RIGHTS 
OF OTHERS.
           
This file may be freely used for commercial and non-commercial applications ONLY 
when used with ImageCraft's ICCV7 for ARM IDE.

If you find a problem with the file, please report it so that it can be fixed.

Modified from lpc213x.h by Pride Embedded and lpc210x.h by Sten Larsson 

Created by Pride Embedded, LLC (Pride@PridEEmbedded.com)

*******************************************************************************/

#define REG32 (volatile unsigned int*)

/*
|----------------------------------------------------
| Vectored Interrupt Controller
|----------------------------------------------------
*/

#define VICIRQStatus    (*(REG32 (0xFFFFF000)))
#define VICFIQStatus    (*(REG32 (0xFFFFF004)))
#define VICRawIntr      (*(REG32 (0xFFFFF008)))
#define VICIntSelect    (*(REG32 (0xFFFFF00C)))
#define VICIntEnable    (*(REG32 (0xFFFFF010)))
#define VICIntEnClear   (*(REG32 (0xFFFFF014)))
#define VICSoftInt      (*(REG32 (0xFFFFF018)))
#define VICSoftIntClear (*(REG32 (0xFFFFF01C)))
#define VICProtection   (*(REG32 (0xFFFFF020)))
#define VICVectAddr     (*(REG32 (0xFFFFF030)))
#define VICDefVectAddr  (*(REG32 (0xFFFFF034)))

#define VICVectAddr0    (*(REG32 (0xFFFFF100)))
#define VICVectAddr1    (*(REG32 (0xFFFFF104)))
#define VICVectAddr2    (*(REG32 (0xFFFFF108)))
#define VICVectAddr3    (*(REG32 (0xFFFFF10C)))
#define VICVectAddr4    (*(REG32 (0xFFFFF110)))
#define VICVectAddr5    (*(REG32 (0xFFFFF114)))
#define VICVectAddr6    (*(REG32 (0xFFFFF118)))
#define VICVectAddr7    (*(REG32 (0xFFFFF11C)))
#define VICVectAddr8    (*(REG32 (0xFFFFF120)))
#define VICVectAddr9    (*(REG32 (0xFFFFF124)))
#define VICVectAddr10   (*(REG32 (0xFFFFF128)))
#define VICVectAddr11   (*(REG32 (0xFFFFF12C)))
#define VICVectAddr12   (*(REG32 (0xFFFFF130)))
#define VICVectAddr13   (*(REG32 (0xFFFFF134)))
#define VICVectAddr14   (*(REG32 (0xFFFFF138)))
#define VICVectAddr15   (*(REG32 (0xFFFFF13C)))

#define VICVectCntl0    (*(REG32 (0xFFFFF200)))
#define VICVectCntl1    (*(REG32 (0xFFFFF204)))
#define VICVectCntl2    (*(REG32 (0xFFFFF208)))
#define VICVectCntl3    (*(REG32 (0xFFFFF20C)))
#define VICVectCntl4    (*(REG32 (0xFFFFF210)))
#define VICVectCntl5    (*(REG32 (0xFFFFF214)))
#define VICVectCntl6    (*(REG32 (0xFFFFF218)))
#define VICVectCntl7    (*(REG32 (0xFFFFF21C)))
#define VICVectCntl8    (*(REG32 (0xFFFFF220)))
#define VICVectCntl9    (*(REG32 (0xFFFFF224)))
#define VICVectCntl10   (*(REG32 (0xFFFFF228)))
#define VICVectCntl11   (*(REG32 (0xFFFFF22C)))
#define VICVectCntl12   (*(REG32 (0xFFFFF230)))
#define VICVectCntl13   (*(REG32 (0xFFFFF234)))
#define VICVectCntl14   (*(REG32 (0xFFFFF238)))
#define VICVectCntl15   (*(REG32 (0xFFFFF23C)))

#define VICIntEnClr     VICIntEnClear
#define VICSoftIntClr   VICSoftIntClear

/*
|----------------------------------------------------
| PCB - Pin Connect Block
|----------------------------------------------------
*/

#define PCB_PINSEL0     (*(REG32 (0xE002C000)))
#define PCB_PINSEL1     (*(REG32 (0xE002C004)))

/*
|----------------------------------------------------
| GPIO - General Purpose I/O
|----------------------------------------------------
*/

#define GPIO_IOPIN      (*(REG32 (0xE0028000))) // Alternate name GPIO = GPIO0
#define GPIO_IOSET      (*(REG32 (0xE0028004)))
#define GPIO_IODIR      (*(REG32 (0xE0028008)))
#define GPIO_IOCLR      (*(REG32 (0xE002800C)))

#define GPIO_IOPIN0     (*(REG32 (0xE0028000))) 
#define GPIO_IOSET0     (*(REG32 (0xE0028004)))
#define GPIO_IODIR0     (*(REG32 (0xE0028008)))
#define GPIO_IOCLR0     (*(REG32 (0xE002800C)))

#define P0_0            0x00000001						 // GPIO short names
#define P0_1            0x00000002
#define P0_2            0x00000004
#define P0_3            0x00000008
#define P0_4            0x00000010
#define P0_5            0x00000020
#define P0_6            0x00000040
#define P0_7            0x00000080
#define P0_8            0x00000100
#define P0_9            0x00000200
#define P0_10           0x00000400
#define P0_11           0x00000800
#define P0_12           0x00001000
#define P0_13           0x00002000
#define P0_14           0x00004000
#define P0_15           0x00008000
#define P0_16           0x00010000
#define P0_17           0x00020000
#define P0_18           0x00040000
#define P0_19           0x00080000
#define P0_20           0x00100000
#define P0_21           0x00200000
#define P0_22           0x00400000
#define P0_23           0x00800000
#define P0_24           0x01000000
#define P0_25           0x02000000
#define P0_26           0x04000000
#define P0_27           0x08000000
#define P0_28           0x10000000
#define P0_29           0x20000000
#define P0_30           0x40000000
#define P0_31           0x80000000

/*
|----------------------------------------------------
| FGPIO - Fast General Purpose I/O
|----------------------------------------------------
*/

#define FGPIO_IODIR     (*(REG32 (0x3FFFC000))) // Alternate name FGPIO = FGPIO0
#define FGPIO_IOMASK    (*(REG32 (0x3FFFC010)))
#define FGPIO_IOPIN     (*(REG32 (0x3FFFC014)))
#define FGPIO_IOSET     (*(REG32 (0x3FFFC018)))
#define FGPIO_IOCLR     (*(REG32 (0x3FFFC01C)))

#define FGPIO_IODIR0    (*(REG32 (0x3FFFC000))) 
#define FGPIO_IOMASK0   (*(REG32 (0x3FFFC010)))
#define FGPIO_IOPIN0    (*(REG32 (0x3FFFC014)))
#define FGPIO_IOSET0    (*(REG32 (0x3FFFC018)))
#define FGPIO_IOCLR0    (*(REG32 (0x3FFFC01C)))

/*
|----------------------------------------------------
| UART0 & UART1
|----------------------------------------------------
*/

// ---- UART 0 --------------------------------------
#define UART0_RBR       (*(REG32 (0xE000C000)))
#define UART0_THR       (*(REG32 (0xE000C000)))
#define UART0_DLL       (*(REG32 (0xE000C000)))
#define UART0_DLM       (*(REG32 (0xE000C004)))
#define UART0_IER       (*(REG32 (0xE000C004)))
#define UART0_IIR       (*(REG32 (0xE000C008)))
#define UART0_FCR       (*(REG32 (0xE000C008)))
#define UART0_LCR       (*(REG32 (0xE000C00C)))
#define UART0_LSR       (*(REG32 (0xE000C014)))
#define UART0_SCR       (*(REG32 (0xE000C01C)))
#define UART0_ACR       (*(REG32 (0xE000C020)))
#define UART0_FDR       (*(REG32 (0xE000C028)))
#define UART0_TER       (*(REG32 (0xE000C030)))

// ---- UART 1 ---------------------------------------
#define UART1_RBR       (*(REG32 (0xE0010000)))
#define UART1_THR       (*(REG32 (0xE0010000)))
#define UART1_DLL       (*(REG32 (0xE0010000)))
#define UART1_DLM       (*(REG32 (0xE0010004)))
#define UART1_IER       (*(REG32 (0xE0010004)))
#define UART1_IIR       (*(REG32 (0xE0010008)))
#define UART1_FCR       (*(REG32 (0xE0010008)))
#define UART1_LCR       (*(REG32 (0xE001000C)))
#define UART1_MCR       (*(REG32 (0xE0010010)))
#define UART1_LSR       (*(REG32 (0xE0010014)))
#define UART1_MSR       (*(REG32 (0xE0010018)))
#define UART1_SCR       (*(REG32 (0xE001001C)))
#define UART1_ACR       (*(REG32 (0xE0010020)))
#define UART1_FDR       (*(REG32 (0xE0010028)))
#define UART1_TER       (*(REG32 (0xE0010030)))

/*
|----------------------------------------------------
| I2C - Inter-Integrated Circuit Bus
|----------------------------------------------------
*/

#define I2C_I2CONSET    (*(REG32 (0xE001C000)))		 // Alternate name I2C = I2C0
#define I2C_I2STAT      (*(REG32 (0xE001C004)))
#define I2C_I2DAT       (*(REG32 (0xE001C008)))
#define I2C_I2ADR       (*(REG32 (0xE001C00C)))
#define I2C_I2SCLH      (*(REG32 (0xE001C010)))
#define I2C_I2SCLL      (*(REG32 (0xE001C014)))
#define I2C_I2CONCLR    (*(REG32 (0xE001C018)))

#define I2C0_I2CONSET    (*(REG32 (0xE001C000)))		
#define I2C0_I2STAT      (*(REG32 (0xE001C004)))
#define I2C0_I2DAT       (*(REG32 (0xE001C008)))
#define I2C0_I2ADR       (*(REG32 (0xE001C00C)))
#define I2C0_I2SCLH      (*(REG32 (0xE001C010)))
#define I2C0_I2SCLL      (*(REG32 (0xE001C014)))
#define I2C0_I2CONCLR    (*(REG32 (0xE001C018)))

#define I2C1_I2CONSET    (*(REG32 (0xE005C000)))		
#define I2C1_I2STAT      (*(REG32 (0xE005C004)))
#define I2C1_I2DAT       (*(REG32 (0xE005C008)))
#define I2C1_I2ADR       (*(REG32 (0xE005C00C)))
#define I2C1_I2SCLH      (*(REG32 (0xE005C010)))
#define I2C1_I2SCLL      (*(REG32 (0xE005C014)))
#define I2C1_I2CONCLR    (*(REG32 (0xE005C018)))

/*
|----------------------------------------------------
| SPI - Serial Peripheral Interface
|----------------------------------------------------
*/

#define SPI_SPCR        (*(REG32 (0xE0020000)))			
#define SPI_SPSR        (*(REG32 (0xE0020004)))
#define SPI_SPDR        (*(REG32 (0xE0020008)))
#define SPI_SPCCR       (*(REG32 (0xE002000C)))
#define SPI_SPINT       (*(REG32 (0xE002001C)))

/*
|----------------------------------------------------
| SSP - Synchronous Serial Port
|----------------------------------------------------
*/

#define SSP_CR0         (*(REG32 (0xE0068000)))
#define SSP_CR1         (*(REG32 (0xE0068004)))
#define SSP_DR          (*(REG32 (0xE0068008)))
#define SSP_SR          (*(REG32 (0xE006800C)))
#define SSP_CPSR        (*(REG32 (0xE0068010)))
#define SSP_IMSC        (*(REG32 (0xE0068014)))
#define SSP_RIS         (*(REG32 (0xE0068018)))
#define SSP_MIS         (*(REG32 (0xE006801C)))
#define SSP_ICR    			(*(REG32 (0xE0068020)))
		
/*
|----------------------------------------------------
| Timer 0 and Timer 1
|----------------------------------------------------
*/															 

// ---- Timer 0 ------------------------------------- 
#define T0_IR           (*(REG32 (0xE0004000)))
#define T0_TCR          (*(REG32 (0xE0004004)))
#define T0_TC           (*(REG32 (0xE0004008)))
#define T0_PR           (*(REG32 (0xE000400C)))
#define T0_PC           (*(REG32 (0xE0004010)))
#define T0_MCR          (*(REG32 (0xE0004014)))
#define T0_MR0          (*(REG32 (0xE0004018)))
#define T0_MR1          (*(REG32 (0xE000401C)))
#define T0_MR2          (*(REG32 (0xE0004020)))
#define T0_MR3          (*(REG32 (0xE0004024)))
#define T0_CCR          (*(REG32 (0xE0004028)))
#define T0_CR0          (*(REG32 (0xE000402C)))
#define T0_CR1          (*(REG32 (0xE0004030)))
#define T0_CR2          (*(REG32 (0xE0004034)))
#define T0_CR3          (*(REG32 (0xE0004038)))
#define T0_EMR          (*(REG32 (0xE000403C)))
#define T0_CTCR         (*(REG32 (0xE0004070)))
#define T0_PWMCON       (*(REG32 (0xE0004074)))


// ---- Timer 1 -------------------------------------
#define T1_IR           (*(REG32 (0xE0008000)))
#define T1_TCR          (*(REG32 (0xE0008004)))
#define T1_TC           (*(REG32 (0xE0008008)))
#define T1_PR           (*(REG32 (0xE000800C)))
#define T1_PC           (*(REG32 (0xE0008010)))
#define T1_MCR          (*(REG32 (0xE0008014)))
#define T1_MR0          (*(REG32 (0xE0008018)))
#define T1_MR1          (*(REG32 (0xE000801C)))
#define T1_MR2          (*(REG32 (0xE0008020)))
#define T1_MR3          (*(REG32 (0xE0008024)))
#define T1_CCR          (*(REG32 (0xE0008028)))
#define T1_CR0          (*(REG32 (0xE000802C)))
#define T1_CR1          (*(REG32 (0xE0008030)))
#define T1_CR2          (*(REG32 (0xE0008034)))
#define T1_CR3          (*(REG32 (0xE0008038)))
#define T1_EMR          (*(REG32 (0xE000803C)))
#define T1_CTCR         (*(REG32 (0xE0008070)))
#define T1_PWMCON       (*(REG32 (0xE0008074)))

/*
|----------------------------------------------------
| Timer 2 and Timer 3
|----------------------------------------------------
*/															 

// ---- Timer 2 ------------------------------------- 
#define T2_IR           (*(REG32 (0xE0070000)))
#define T2_TCR          (*(REG32 (0xE0070004)))
#define T2_TC           (*(REG32 (0xE0070008)))
#define T2_PR           (*(REG32 (0xE007000C)))
#define T2_PC           (*(REG32 (0xE0070010)))
#define T2_MCR          (*(REG32 (0xE0070014)))
#define T2_MR0          (*(REG32 (0xE0070018)))
#define T2_MR1          (*(REG32 (0xE007001C)))
#define T2_MR2          (*(REG32 (0xE0070020)))
#define T2_MR3          (*(REG32 (0xE0070024)))
#define T2_CCR          (*(REG32 (0xE0070028)))
#define T2_CR0          (*(REG32 (0xE007002C)))
#define T2_CR1          (*(REG32 (0xE0070030)))
#define T2_CR2          (*(REG32 (0xE0070034)))
#define T2_EMR          (*(REG32 (0xE007003C)))
#define T2_CTCR         (*(REG32 (0xE0070070)))
#define T2_PWMCON       (*(REG32 (0xE0070074)))


// ---- Timer 3 -------------------------------------
#define T3_IR           (*(REG32 (0xE0074000)))
#define T3_TCR          (*(REG32 (0xE0074004)))
#define T3_TC           (*(REG32 (0xE0074008)))
#define T3_PR           (*(REG32 (0xE007400C)))
#define T3_PC           (*(REG32 (0xE0074010)))
#define T3_MCR          (*(REG32 (0xE0074014)))
#define T3_MR0          (*(REG32 (0xE0074018)))
#define T3_MR1          (*(REG32 (0xE007401C)))
#define T3_MR2          (*(REG32 (0xE0074020)))
#define T3_MR3          (*(REG32 (0xE0074024)))
#define T3_CCR          (*(REG32 (0xE0074028)))
#define T3_CR0          (*(REG32 (0xE007402C)))
#define T3_CR1          (*(REG32 (0xE0074030)))
#define T3_CR2          (*(REG32 (0xE0074034)))
#define T3_EMR          (*(REG32 (0xE007403C)))
#define T3_CTCR         (*(REG32 (0xE0074070)))
#define T3_PWMCON       (*(REG32 (0xE0074074)))

/*
|----------------------------------------------------
| RTC - Real Time Clock
|----------------------------------------------------
*/	

// ---- RTC: Miscellaneous Register Group -----------
#define RTC_ILR         (*(REG32 (0xE0024000)))
#define RTC_CTC         (*(REG32 (0xE0024004)))
#define RTC_CCR         (*(REG32 (0xE0024008)))  
#define RTC_CIIR        (*(REG32 (0xE002400C)))
#define RTC_AMR         (*(REG32 (0xE0024010)))
#define RTC_CTIME0      (*(REG32 (0xE0024014)))
#define RTC_CTIME1      (*(REG32 (0xE0024018)))
#define RTC_CTIME2      (*(REG32 (0xE002401C)))

// ---- RTC: Timer Control Group --------------------
#define RTC_SEC         (*(REG32 (0xE0024020)))
#define RTC_MIN         (*(REG32 (0xE0024024)))
#define RTC_HOUR        (*(REG32 (0xE0024028)))
#define RTC_DOM         (*(REG32 (0xE002402C)))
#define RTC_DOW         (*(REG32 (0xE0024030)))
#define RTC_DOY         (*(REG32 (0xE0024034)))
#define RTC_MONTH       (*(REG32 (0xE0024038)))
#define RTC_YEAR        (*(REG32 (0xE002403C)))

// ---- RTC: Alarm Control Group ---------------------
#define RTC_ALSEC       (*(REG32 (0xE0024060)))
#define RTC_ALMIN       (*(REG32 (0xE0024064)))
#define RTC_ALHOUR      (*(REG32 (0xE0024068)))
#define RTC_ALDOM       (*(REG32 (0xE002406C)))
#define RTC_ALDOW       (*(REG32 (0xE0024070)))
#define RTC_ALDOY       (*(REG32 (0xE0024074)))
#define RTC_ALMON       (*(REG32 (0xE0024078)))
#define RTC_ALYEAR      (*(REG32 (0xE002407C)))

// ---- RTC: Reference Clock Divider Group -----------
#define RTC_PREINT      (*(REG32 (0xE0024080)))
#define RTC_PREFRAC     (*(REG32 (0xE0024084)))

/*
|----------------------------------------------------
| WD - Watchdog
|----------------------------------------------------
*/	

#define WD_WDMOD        (*(REG32 (0xE0000000)))
#define WD_WDTC         (*(REG32 (0xE0000004)))
#define WD_WDFEED       (*(REG32 (0xE0000008)))
#define WD_WDTV         (*(REG32 (0xE000000C)))

/*
|----------------------------------------------------
| AD0 - Analog to Digital Converter
|----------------------------------------------------
*/	

// ---- AD 0 -------------------------------------
#define AD_CR     		(*(REG32 (0xE0034000)))		 		// Alternate name AD = AD0
#define AD_GDR      	(*(REG32 (0xE0034004)))
#define AD_STAT      	(*(REG32 (0xE0034030)))
#define AD_INTEN     	(*(REG32 (0xE003400C)))

#define AD_DR0      	(*(REG32 (0xE0034010)))
#define AD_DR1      	(*(REG32 (0xE0034014)))
#define AD_DR2      	(*(REG32 (0xE0034018)))
#define AD_DR3      	(*(REG32 (0xE003401C)))
#define AD_DR4      	(*(REG32 (0xE0034020)))
#define AD_DR5      	(*(REG32 (0xE0034024)))
#define AD_DR6      	(*(REG32 (0xE0034028)))
#define AD_DR7      	(*(REG32 (0xE003402C)))

#define AD0_CR     		(*(REG32 (0xE0034000)))		 		
#define AD0_GDR      	(*(REG32 (0xE0034004)))
#define AD0_STAT     	(*(REG32 (0xE0034030)))
#define AD0_INTEN    	(*(REG32 (0xE003400C)))

#define AD0_DR0      	(*(REG32 (0xE0034010)))
#define AD0_DR1      	(*(REG32 (0xE0034014)))
#define AD0_DR2      	(*(REG32 (0xE0034018)))
#define AD0_DR3      	(*(REG32 (0xE003401C)))
#define AD0_DR4      	(*(REG32 (0xE0034020)))
#define AD0_DR5      	(*(REG32 (0xE0034024)))
#define AD0_DR6      	(*(REG32 (0xE0034028)))
#define AD0_DR7      	(*(REG32 (0xE003402C)))


/*
|----------------------------------------------------
| SCB - System Control Block
|----------------------------------------------------
*/	

#define SCB_EXTINT      (*(REG32 (0xE01FC140)))
#define SCB_EXTWAKE     (*(REG32 (0xE01FC144)))
#define SCB_EXTMODE     (*(REG32 (0xE01FC148)))
#define SCB_EXTPOLAR    (*(REG32 (0xE01FC14C)))

#define SCB_MEMMAP      (*(REG32 (0xE01FC040)))

#define SCB_PLLCON      (*(REG32 (0xE01FC080)))
#define SCB_PLLCFG      (*(REG32 (0xE01FC084)))
#define SCB_PLLSTAT     (*(REG32 (0xE01FC088)))
#define SCB_PLLFEED     (*(REG32 (0xE01FC08C)))

#define SCB_PCON        (*(REG32 (0xE01FC0C0)))
#define SCB_PCONP       (*(REG32 (0xE01FC0C4)))

#define SCB_APBDIV      (*(REG32 (0xE01FC100)))
#define SCB_VPBDIV      (*(REG32 (0xE01FC100)))

#define SCB_RSIR        (*(REG32 (0xE01FC180)))
#define SCB_CSPR       	(*(REG32 (0xE01FC184)))
#define SCB_SCS      		(*(REG32 (0xE01FC1A0)))

/*
|----------------------------------------------------
| MAM - Memory Acceleration Module
|----------------------------------------------------
*/	

#define MAM_CR					(*(REG32 (0xE01FC000)))
#define MAM_TIM					(*(REG32 (0xE01FC004)))


#endif 																							// end lpc2103_h 


