//Project:    STX ARM7 experimental LED display
//Chip:       LPC2101
//Crystal:    14.7456MHz
//Core Freq:  58.982400MHz
//PCLK:       58.982400MHz
//Modes:      Supervisor, IRQ
//Stack Size: IRQ=200bytes

#include <ARM/philips/lpc2103.h>
#include <arm_macros.h>

#include "font.c"
#include "adc.c"

////// Prototypes ////////
//////// IRQ ///////////////
void timer0_isr(void);
void timer1_isr(void);
void uart0_isr(void);
void i2c_isr(void);
void default_vector(void);
///////////////////////////
void shift_left(void);
void print_time(void);
char change_time(void);

#define RESET_TRUE	  1
#define RESET_FALSE	  0
char read_string(char reset);
//void cls(void);
void uart_putchar(char data);
void uart_putsf(const char *string);
char uart_getchar(void);
unsigned short adc_read(char ch);
/// I2C functions ///
#define EE_ADRESS 0xA0
#define EE_READ	  1
#define EE_WRITE  0
void i2c_init(void);
void mem_write(unsigned short adress, char data);
void mem_read(unsigned short adress);
/////////////////////

#define THRE 5
#define TEMT 6

#define TRUE 1
#define FALSE 0

///// Global Vars ///////
#define SCREEN_LEN 64
#define SCR_MAX_CNT (SCREEN_LEN-1)
unsigned char video_ram[SCREEN_LEN];//screen data SCREEN_LEN
const char logo_string[]=" ELECTRONICS.UZ          ARM7TDMI-S   Designed by Sergey :)  \n";
char uart_data;
char uart_full=FALSE;
char instant_str[1000];//string for instant message
char clock_en;//enable/disable clock flag
/////////////////////////////////////////////////////////////////////
#define STR_EEPROM	0
#define STR_INSTANT	1
#define STR_LOGO	2

char str_mode=STR_EEPROM;//read string from EEPROM
////////////////////////// i2c memory ///////////////////////////////////
unsigned char i2c_msg[3];
unsigned char i2c_mem_wr;//memory write flag
unsigned char i2c_lock=FALSE;//I2C busy flag
unsigned char i2c_max_cnt;
unsigned char i2c_nack;//NACK flag 
volatile char i2c_out;//received data from I2DAT

void i2c_init(void)
{
PCB_PINSEL0=PCB_PINSEL0|(1<<4)|(1<<6);//connect i2c
I2C0_I2CONCLR=0x7C;//clear all bits
I2C0_I2CONSET=0x40;//i2c enable
I2C0_I2SCLH=73;
I2C0_I2SCLL=73;//100KHz
}

/*void mem_reset(void)
{
 I2C0_I2CONSET=0x40|0x20;//I2EN+STA
 i2c_wait_ready();
 I2C0_I2CONCLR=0x20;
 I2C0_I2DAT=0xFF;
 i2c_wait_ready();
 I2C0_I2CONSET=0x40|0x10;//I2EN+STO
}*/


void mem_write(unsigned short adress, char data)
{
 i2c_msg[0]=(char)(adress>>8);
 i2c_msg[1]=(char)adress;
 i2c_msg[2]=data;
 i2c_mem_wr=TRUE;
 i2c_lock=TRUE;
 i2c_max_cnt=3;
 I2C0_I2CONSET=0x20;//STA
 while(i2c_lock==TRUE);
 /////// MEM ACK Polling ///////
 i2c_mem_wr=FALSE;
 i2c_max_cnt=2;
 i2c_nack=TRUE;
 I2C0_I2CONSET=0x20;//STA
 while(i2c_nack==TRUE)
    {
	i2c_lock=TRUE;
	I2C0_I2CONSET=0x20;//STA
	while(i2c_lock==TRUE);
	};
}

void mem_read(unsigned short adress)
{
 while(i2c_lock==TRUE);
 i2c_msg[0]=(char)(adress>>8);
 i2c_msg[1]=(char)adress;
 i2c_mem_wr=FALSE;
 i2c_max_cnt=2;
 i2c_lock=TRUE;
 I2C0_I2CONSET=0x40|0x20;//I2EN+STA
} 
/////////////////////////////////////////////////////////////////////

/////////////////// Screen Functions ////////////////////////////////
void scr_putchar(char chr)
{
unsigned char i;
unsigned char len=0;
unsigned short data_ptr=0;

			data_ptr=charLUT[chr];
			len=char_set[data_ptr];//get char width

			for(i=0;i<len;i++)
							  {
							  shift_left();
							  video_ram[SCR_MAX_CNT]=char_set[++data_ptr];
							  };
			shift_left();
			video_ram[SCR_MAX_CNT]=0x00;//blank separator
}

/* Shift screen left for one column */
void shift_left(void)
{
unsigned char i;

for(i=0;i<SCR_MAX_CNT;i++)
				 {
				 video_ram[i]=video_ram[i+1];
				 };
}
/////////////////////////////////////////////////////////////////////

/* Timer0 interrupt handler*/
#pragma interrupt_handler timer0_isr
void timer0_isr(void)
{//   
static unsigned char str_cnt=0x01;
static unsigned char str_cnt1=0x40;
unsigned char i;

for(i=0;i<SCREEN_LEN;i++)//issue current string
				 {
				 FGPIO_IOCLR=0x00000090;//clear SH_CP & ST_CP bits 
				 if(video_ram[i] & str_cnt)//if bit set
				    FGPIO_IOSET=0x00000040;//P0.6=1
				 else
				 	FGPIO_IOCLR=0x00000040;//=0
				 asm("nop");
				 FGPIO_IOSET=0x00000010;//set SH_CP/clock sygnal
				 asm("nop"); 
				 };
FGPIO_IOSET=0x00000080;//set ST_CP P0.7
FGPIO_IOCLR=0x0000FF00;//clear string selector bits P0.8-P0.15
FGPIO_IOSET=(str_cnt1)<<7;//set next string 
str_cnt<<=1;
str_cnt1>>=1;
if(str_cnt>0x40)
			{
			str_cnt=0x01;
			str_cnt1=0x80;
			}

T0_IR=0x00000001;//clear MR0 interrupt flag
VICVectAddr = 0; // Acknowledge Interrupt
}

/* Timer1 interrupt handler */
#pragma interrupt_handler timer1_isr
void timer1_isr(void)
{//   
static unsigned char width_cnt=0;
static unsigned char width_cnt_max=0;
static char current_symb;
unsigned int adc_result;
static unsigned short data_ptr=0;

adc_result=adc_read(ADC_CH2);
T1_TC=0x00;
T1_MR0=184320+(adc_result<<11);//set speed from ADC
T1_IR=0x00000001;//clear MR0 flag

shift_left();
if(width_cnt==width_cnt_max)//load new char
				   {
				   video_ram[SCR_MAX_CNT]=0x00;//blank separator
				   current_symb=read_string(RESET_FALSE);
				   width_cnt=0x00;
				   data_ptr=charLUT[current_symb];
				   width_cnt_max=char_set[data_ptr];//get char width 
				   VICVectAddr=0;//return
				   return;
				   };

video_ram[SCR_MAX_CNT]=char_set[++data_ptr];
++width_cnt;
				   
VICVectAddr=0;//return
}

/* UART0 interrupt handler */
#pragma interrupt_handler uart0_isr
void uart0_isr(void)
{
 char irq_id, irq_stat;
 
 irq_id=UART0_IIR;//IRQ identification
 irq_stat=UART0_LSR;//IRQ status
 
 uart_data=UART0_RBR;
 if(!(irq_stat & 0x80))//if no errors
    {
	if(irq_stat & 0x01)//RDR
	   {
	   uart_full=TRUE;
	   uart_putchar(uart_data);//return echo
	   }
	}
 
 VICVectAddr = 0; // Acknowledge Interrupt
}

/* I2C0 Interrupt Handler*/
#pragma interrupt_handler i2c_isr
void i2c_isr(void)
{
 static unsigned char i2c_cnt;
 
 switch(I2C0_I2STAT)
 					{
					case 0x08://start
						 I2C0_I2CONCLR=0x20|0x10;//clear STA+STO
						 I2C0_I2DAT=EE_ADRESS|EE_WRITE;//SLA+W
						 i2c_cnt=0;
						 break;
					case 0x10://restart
						 I2C0_I2CONCLR=0x20;//clear STA
						 I2C0_I2DAT=EE_ADRESS|EE_READ;
						 break;
					case 0x18://SLA+W,ACK
						 I2C0_I2DAT=i2c_msg[i2c_cnt++];
						 i2c_nack=FALSE;//memory answered
						 break;
					case 0x20://SLA+W, NACK
						 i2c_nack=TRUE;//memory not ack
						 i2c_lock=FALSE;
						 I2C0_I2CONSET=0x10;//STO
						 break;
					case 0x28://data transmitted, ACK
						 //uart_putsf("TX,ACK ");
						 if(i2c_cnt<i2c_max_cnt)
						 	 {I2C0_I2DAT=i2c_msg[i2c_cnt++];}
						 else
						 	 {
							 if(i2c_mem_wr==FALSE)
							 	 {I2C0_I2CONSET=0x20;}//restart
							 else//i2c_mem_wr=TRUE
							 	 {
								 I2C0_I2CONSET=0x10;//STO
								 i2c_lock=FALSE;
								 };
							 };
						break;
					case 0x58://data received, NACK
						 I2C0_I2CONSET=0x10;//STO
						 i2c_lock=FALSE;//by default bit AA in I2CONSET cleared->always NACK
						 i2c_out=I2C0_I2DAT;
						 break;
					};
 I2C0_I2CONCLR=0x08;//clear SI
 VICVectAddr=0;
}

/* Non-vectored interrupt */
#pragma interrupt_handler default_vector
void default_vector(void)
{
VICVectAddr = 0; // Acknowledge Interrupt
}

void main()
{
char data;
unsigned short entered_chr;
///// MAM init //////
MAM_TIM=0x01;//0x00000003;//MAM fetch cycles are 3CCLK in duration
MAM_CR=0x02;//MAM fully enabled

/*//// PLL init //////
SCB_PLLCFG=0x00000024;//M=4, P=2, Fcco=235,9MHz
SCB_PLLCON=0x00000001;//enable PLL
SCB_PLLFEED=0xAA;
SCB_PLLFEED=0x55;//enable changes
while(!(SCB_PLLSTAT & 0x400));//wait for PLL stabilization
SCB_PLLCON=0x00000003;//connect PLL
SCB_PLLFEED=0xAA;
SCB_PLLFEED=0x55;//apply settings*/

////// Peripheral clock /////
SCB_VPBDIV=0x00000001;//PCLK=CCLK/1

///// Port init /////

SCB_SCS=0x01;//Enable Fast GPIO
//GPIO_IODIR=0x0000FFD0;
FGPIO_IODIR=0x0000FFD0;
//P0.4=SH_CP
//P0.6=DS
//P0.7=ST_CP
//P0.8-P0.15=string selector

///// UART0 init //////
PCB_PINSEL0=PCB_PINSEL0 | 0x05;//enable TXD0, RXD0 lines
UART0_FCR=0x07;//enable and reset RX/TX FIFOs, trigger level0
UART0_LCR=0x83;//DLAB=1, 8N1

UART0_DLL=0x60;//0x80;//baudrate=9600
UART0_DLM=0x00;//0x01;

UART0_LCR=0x03;//DLAB=0
UART0_TER=0x80;//TXEN
UART0_IER=0x01;//RDA interrupt enable

///// VIC init /////
VICIntSelect=0x00000000;//all requests are IRQ
VICSoftInt=0x00000000;
VICSoftIntClear=~VICSoftInt;//clear all interrupts
//Interrupt adresses
VICVectAddr0=(unsigned)timer0_isr;//highest priority vectored slot
VICVectAddr1=(unsigned)timer1_isr;
VICVectAddr2=(unsigned)uart0_isr;
VICVectAddr3=(unsigned)i2c_isr;
VICDefVectAddr=(unsigned)default_vector;//non-vectored interrupt
VICVectCntl0=0x20 | 0x04;//enable slot and assign TIMER0 channel
VICVectCntl1=0x20 | 0x05;//enable slot + TIMER1
VICVectCntl2=0x20 | 0x06;//UART0
VICVectCntl3=0x20 | 0x09;//I2C0
VICIntEnable=0x70|(1<<9);//enable TIMER0, TIMER1, UART0 channel, I2C0

///// Timer0 init ///////
T0_MCR=0x01 | 0x02;//MR0 int + MR0 reset
T0_MR0=7372;//2000Hz//18432;//73727;//match value, freq=800Hz,   800/8=100
T0_TCR=0x01;//count enable

///// Timer1 init ////////
T1_MCR=0x01|0x02;//MR0 int + MR0 reset
T1_MR0=1474560;//5898239;//freq=10Hz,   
T1_TCR=0x01;//count enable

////////// RTC /////////////
RTC_CCR=0x10|0x01;//Clock enable, 32KHz

i2c_init();
//mem_reset();
read_string(RESET_TRUE);

__ENABLE_INTERRUPT();
//////////////////////////////////
mem_read(8191);//load clock status
while(i2c_lock==TRUE);
clock_en=i2c_out;
/////////////////////////////////
uart_putsf("***STX_ARM Power-up***\x0a\x0d");

while(1)//loop forever
   {
   if(uart_full==TRUE)
     {
	 uart_full=FALSE;
	 if(uart_data==0x0D)//Enter pressed
	   {
	   while(uart_full==FALSE);
	   if(uart_data==0x0D)//Enter again pressed
	      {//go to the main menu
	      uart_putsf("\x0a\x0d***STX ARM Main menu***\x0a\x0d"); 
		  uart_putsf("1- Enter instant text\x0a\x0d");
		  uart_putsf("2- Enter EEPROM text\x0a\x0d");
		  uart_putsf("3- Reset\x0a\x0d");
		  if(clock_en)//clock_en==TRUE
		     {uart_putsf("4- Disable clock\x0a\x0d");}
		  else
		     {uart_putsf("4- Enable clock\x0a\x0d");};
		  uart_putsf("5- Change time\x0a\x0d"); 
		  uart_putsf("6- Exit\x0a\x0d"); 
	get_choice:
		  uart_putsf("Your choice:");
		  data=uart_getchar();
		  uart_putsf("\x0a\x0d");
		  switch(data)
		     {
			 case '1'://instant editor
			 	uart_putsf("Enter instant text and press [ENTER]\x0a\x0d");
				str_mode=STR_LOGO;
				read_string(RESET_TRUE);
				data='';
				uart_data='';
				entered_chr=0;
				uart_full=FALSE;
				while((entered_chr<=999)&&(data!=0x0D))
				     {
					 if(uart_full==TRUE)
					      {
						  instant_str[entered_chr]=uart_data;
						  data=uart_data;
						  ++entered_chr;
						  uart_full=FALSE;
						  }
					 };
				if(entered_chr<2)
				     {
					 uart_putsf("\x0a\x0d");
					 uart_putsf("Too few symbols!\x0a\x0d");
					 }
				else
				     {
					 if(entered_chr==1000)
					    instant_str[--entered_chr]='\x0d';//always add Enter
					 uart_putsf("\x0a\x0d");
					 uart_putsf("Instant string saved\x0a\x0d");
					 read_string(RESET_TRUE);//reset pointer
					 str_mode=STR_INSTANT;
					 }
			    break;
			 case '2'://EEPROM editor
			 	uart_putsf("Enter EEPROM text and press [ENTER]\x0a\x0d");
				str_mode=STR_LOGO;
				read_string(RESET_TRUE);				
				uart_data='';
				entered_chr=0;
				uart_full=FALSE;
				while(entered_chr<=8190)
				     {
					 if(uart_full==TRUE)
					      {
						  if(uart_data==0x0D)//ENTER
						  							break;
						  uart_full=FALSE;
						  mem_write(entered_chr,uart_data);
						  ++entered_chr;
						  }
					 };
				if(entered_chr<2)
				     {
					 uart_putsf("\x0a\x0d");
					 uart_putsf("Too few symbols!\x0a\x0d");
					 }
				else
				     {
					 uart_putsf("\x0a\x0d");
					 uart_putsf("EEPROM string saved\x0a\x0d");
					 str_mode=STR_EEPROM;
					 read_string(RESET_TRUE);
					 }
				mem_write(entered_chr,0x0D);//add ENTER			 	
			 	break;
			 case '3'://reset
			    uart_putsf("Reset\x0a\x0d");
				read_string(RESET_TRUE);//reset pointer
				str_mode=STR_EEPROM;
			    break;
			 case '4'://enable/disable clock
			    uart_putsf("\x0a\x0d");
			 	if(clock_en)//==TRUE
									{
									mem_write(8191,FALSE);
									uart_putsf("Clock disabled");
									clock_en=FALSE;
									}
				else
									{
									mem_write(8191,TRUE);
									uart_putsf("Clock enabled");
									clock_en=TRUE;
									};
				break;
			 case '5'://change time
			    if (change_time()==TRUE)//error
				   {
				   uart_putsf("Error. Incorrect time format!");
				   }
				else
					{
					uart_putsf("Time was changed");
					};
				break;
			 case '6'://exit
			    uart_putsf("\x0a\x0d Exit \x0a\x0d");
			    break;
			 default:
			    goto get_choice;
			 }
		  uart_putsf("\x0a\x0d Terminated \x0a\x0d");
		  uart_full=FALSE;
	      }
	   };
	 }
   };
}

/* Clear screen */
/*void cls(void)
{
 char i;
 for(i=0;i<SCREEN_LEN;i++)
 				  {
				  video_ram[i]=0x00;
				  };
}*/

/* Read the next char from text string */
char read_string(char reset)
{
static unsigned short c_pointer=0;
char data;

if(reset) 
 {//reset pointer
 c_pointer=0;
 return;
 }; 

switch(str_mode)
				{
				case STR_EEPROM:
					 data=i2c_out;
					 //mem_read(c_pointer);
					 break;
				case STR_INSTANT:
					 data=instant_str[c_pointer];//instant message
					 break;
				case STR_LOGO:
					 data=logo_string[c_pointer];//message from flash
					 break;
				} 
 
if((data==0x0D)||(data=='\n'))//end of string
							  {
							  c_pointer=0;
							  if(clock_en)//clock_en==TRUE
							  					{print_time();};
							  if(str_mode==STR_INSTANT){--c_pointer;};
							  data=' ';
							  };
if(str_mode==STR_EEPROM) {mem_read(c_pointer);};
++c_pointer;
return data;
}

/////////////////////////////// UART Functions ////////////////////////////
void uart_putchar(char data)//send single char
{
 while(!(UART0_LSR & (1<<THRE)));//wait while TX fifo busy
 UART0_THR=data;//send new char
}

void uart_putsf(const char *string)//send null-terminated string from FLASH
{
 char k;
 
 while(k=*string++) uart_putchar(k);
}

char uart_getchar(void)
{
   uart_full=FALSE;
   while(uart_full==FALSE);
   uart_full=FALSE;
   return uart_data;
}
///////////////////////////////////////////////////////////////////////////

///////////////////////// RTC Time Functions //////////////////////////////
void print_time(void)
{
 unsigned char hour, min;
 unsigned char h1,h2,m1,m2;
 unsigned int time;
 
 time=RTC_CTIME0;
 hour=(time>>16) & 0x1F;
 min=(time>>8) & 0x3F;
 
 h1='0'; h2='0'; m1='0'; m2='0';
 while(hour>=10){hour-=10;++h1;};
 while(hour)    {--hour;  ++h2;};
 while(min>=10) {min-=10; ++m1;};
 while(min)     {--min;   ++m2;};
 
 if(h1=='0') h1=' ';//space
 
 scr_putchar(' ');//space
 scr_putchar(h1);
 scr_putchar(h2);
 scr_putchar(':');
 scr_putchar(m1);
 scr_putchar(m2);
 shift_left();
}

char change_time(void)
{
 unsigned char t[4], i;
 unsigned char hour, min;
 
 uart_putsf("\x0a\x0d");
 uart_putsf("Enter time in format HH:MM\x0a\x0d");
 
 t[0]=uart_getchar();
 t[1]=uart_getchar();
 uart_putchar(':');
 t[2]=uart_getchar();
 t[3]=uart_getchar();
 
 uart_putsf("\x0a\x0d");

 for (i=0;i<4;i++)
 	 {
	 if((t[i]<'0')||(t[i]>'9'))//check for error
	 								   {
									   return 1;//error
									   }; 
	 };

 hour=(t[0]-'0')*10+(t[1]-'0');
 min= (t[2]-'0')*10+(t[3]-'0');
 
 if((hour>23)||(min>59))
 						{
						return 1;//error
						}
  
 //// Set new time ////
 RTC_CCR=(1<<0)|(1<<1)|(1<<4);//CTC reset
 RTC_SEC=0;
 RTC_MIN=min;
 RTC_HOUR=hour; 
 RTC_CCR=(1<<0)|(1<<4);//Start
 
 return 0;//no errors
}
///////////////////////////////////////////////////////////////////////////
unsigned short adc_read(char ch)
{
 PCB_PINSEL1|=(0x03<<16);//enable AD0.2 input
 AD0_CR=ch|CR_CLKDIV|CR_PDN|CR_START_NOW;//select channel and start
 while(!(AD0_GDR & GDR_DONE));//wait for completing conversion
 return ((AD0_GDR>>6)&0x3FF);
}